/*:ts=8*/
/*****************************************************************************
 * FIDOGATE --- Gateway software UNIX <-> FIDO
 *
 * $Id: config.h,v 3.7 1994/05/02 05:36:42 mj Exp mj $
 *
 * Configuration header file
 *
 *****************************************************************************
 * Copyright (C) 1990, 1993
 *  _____ _____
 * |     |___  |   Martin Junius             FIDO:      2:242/6.1
 * | | | |   | |   Republikplatz 3           Internet:  mj@sungate.fido.de
 * |_|_|_|@home|   D-52072 Aachen, Germany   Phone:     ++49-241-86931 (voice)
 *
 * This file is part of FIDOGATE.
 *
 * FIDOGATE is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * FIDOGATE is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with FIDOGATE; see the file COPYING.  If not, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************/

/*
 * Address configuration now in ../lib/config !!!
 */


/***** General configuration *************************************************/

/** Domain suffix for standard FIDO addresses. DO NOT CHANGE THIS! **/
#define FIDODOMAIN	".fidonet.org"

/** Postmaster name **/
#define POSTMASTER	"postmaster"

/*
 * Enable conversion of the addresses generated by the MAUS <-> FIDO
 * gateway to their Internet form, e.g.
 *     `User Name % MAUS AC'  -->  `user_name@ac.maus.de'
 * and vice versa.
 *
 * This requires the MAUS config file in LIBDIR.
 */
#define MAUS_DE

#ifdef MAUS_DE
# define MAUS_DE_FROM			/* MAUS handling in fido2rfc */
# define MAUS_DE_TO			/* MAUS handling in rfc2fido */
# define MAUS_STRING	"_%_MAUS_"	/* For addressing gateway */
#endif

/*
 * Only allow mail from/to nodes registered in HOSTS
 */
#define HOSTS_RESTRICTED

/*
 * Generate local FTN addresses, e.g.
 *     user_name%p.f.n.z@host.domain
 * instead of
 *     user_name@p.f.n.z.domain
 */
/* #define LOCAL_FTN_ADDRESSES */

/*
 * Send all Internet mail via FTN and a FTN-Internet gateway.
 * This allows to use FIDOGATE in an FTN point setup.
 */
/* #define ROUTE_VIA_FTN_GATEWAY */



/***** fido2rfc configuration ************************************************/

/** Add X-FTN-Tearline header for tear line **/
#define X_FTN_TEARLINE
/** Add X-FTN-Via headers for ^AVia lines **/
#define X_FTN_VIA
/** Add X-FTN-Path headers for ^APath lines **/
#define X_FTN_PATH
/** Add X-FTN-Origin header for * Origin line **/
#define X_FTN_ORIGIN

/** Junk newsgroup for areas with unknown area<->newsgroup conversion **/
#define FTN_JUNK	"fido.junk"

/** Errors-To: return address for bounced mail **/
/* #define ERRORS_TO	    "postmaster@sungate.fido.de" */

/** Add `@host.domain' to local addresses **/
#define ADD_DOMAIN



/***** rfc2fido configuration ************************************************/

/** Passthru operation for NetMail: FIDO->Internet->FIDO **/
#define PASSTHRU_NETMAIL
/** Passthru operation for EchoMail: FIDO->Internet->FIDO **/
#define PASSTHRU_ECHOMAIL /* Not yet implemented */

/*
 * Type of 4D EchoMail control lines.
 *
 * With this enabled FIDOGATE generates SEEN-BY and ^APATH like:
 *     SEEN-BY: 242/6.1
 *     ^APATH: 242/6.1
 * Else (undef'ed) only the boss node's address is used:
 *     SEEN-BY: 242/6
 *     ^APATH: 242/6
 * There seems to be no standard for 4D EchoMail addressing. If you run
 * into problems with this kind of EchoMail message, please change the
 * code in rfc2fido.c:print_origin().
 */
/* #define FTN_4D_ECHOMAIL_WITH_POINTS */

/*
 * Packet password (max. 8 characters, "" = empty)
 */
#define PACKET_PASSWD	""

/*
 * Product code for packets generated by FIDOGATE, 0xfe is used because
 * this code is reserved for new products, when the code numbers ran out.
 */
#define PRODUCT_CODE	0xfe

/*
 * Enable this when running FIDOGATE as a FIDO
 * point without Internet connectivity!!!
 */
/* #define NO_FROM_LINE */

/*
 * Enable support for the FSC-0035 kludges, i.e. ^AREPLYADDR and ^AREPLYTO
 */
#ifndef NO_FROM_LINE
# define FSC_0035
#endif

/*
 * Always generate a ^AINTL kludge for NetMails. (undef: only for mails
 * to other zones)
 */
#define FTN_FORCE_INTL



/*
 * Files in LIBDIR
 */
#define CONFIG		"config"	/* Main FIDOGATE config file */
#define HOSTS		"hosts"		/* Host file (Fido.DE) */
#define ALIASES		"aliases"	/* User aliases file */
#define AREAS		"areas"		/* Areas <-> Newsgroups */
#define MAUS		"maus"		/* MAUSNET domain list */

#define LOG		"log"		/* Log file */

#define SEQ_MAIL	"seq.mail"	/* fido2rfc: mail files */
#define SEQ_NEWS	"seq.news"	/* fido2rfc: news files */
#define SEQ_MSGID	"seq.msgid"	/* rfc2fido: ^AMSGID */
#define SEQ_PKT		"seq.pkt"	/* rfc2fido: outgoing packets */
#define SEQ_FF		"seq.ff"	/* ffx: control/data files */

/*
 * Directories in SPOOLDIR
 */
#define INBOUND		"in"		/* Input for fido2rfc */
#define INBOUND_MAIL	"mail"		/* in/mail output from fido2rfc */
#define INBOUND_NEWS	"news"		/* in/news Output from fido2rfc */
#define UNSECURE	"unsecure"	/* Unsecure input for fido2rfc */
#define OUTBOUND	"out"		/* Output of rfc2fido */

/*
 * Secure permissions
 */
/* #define SECURE */

#ifdef SECURE		/* Secure permissions */
# define PACKET_MODE	0600		/* Mode for outbound packets */
# define BSY_MODE	0644		/* Mode for BSY files */
# define FLO_MODE	0644		/* Mode for FLO files */
# define DATA_MODE	0600		/* Mode for ffx data files */
#else			/* Open permissions */
# define PACKET_MODE	0666		/* Mode for outbound packets */
# define BSY_MODE	0666		/* Mode for BSY files */
# define FLO_MODE	0666		/* Mode for FLO files */
# define DATA_MODE	0666		/* Mode for ffx data files */
#endif


/*
 * RFC headers recognized at beginning of FTN message body
 */
#define FTN_RFC_HEADERS \
	"From:",        \
	"Reply-To:",    \
	"UUCPFROM:",    \
	"To:",          \
	"Cc:"


/*
 * Uncomment the following define if you want FLO files for BinkleyTerm
 * in the outbound directory.
 */
/* #define BINKLEY_FLO	".DLO" */



/*
 * System dependend configuration
 *
 *   DO_HAVE_GETTIMEOFDAY	Do you have gettimeofday()?
 *
 *   DO_HAVE_TM_GMTOFF		Does your (struct tm) have a tm_gmtoff field?
 *
 *   DO_HAVE_TM_ZONE		Does your (struct tm) have a tm_zone field?
 *
 *   DO_HAVE_STRFTIME		Do you have strftime()?
 *
 *   DO_HAVE_TZNAME		Do you have extern char *tzname[2]?
 *
 * Define only one of DO_HAVE_TM_ZONE, DO_HAVE_STRFTIME, DO_HAVE_TZNAME!!!
 */

#ifdef sun				/* SUNOS */
# define DO_HAVE_GETTIMEOFDAY
# define DO_HAVE_TM_GMTOFF
/* #define DO_HAVE_TM_ZONE */
# define DO_HAVE_STRFTIME
/* #define DO_HAVE_TZNAME */
#endif

#ifdef __linux__			/* LINUX */
# define DO_HAVE_GETTIMEOFDAY
# undef  DO_HAVE_TM_GMTOFF
/* #define DO_HAVE_TM_ZONE */
# define DO_HAVE_STRFTIME
/* #define DO_HAVE_TZNAME */
#endif


#ifdef DO_HAVE_TM_GMTOFF
# undef  DONT_HAVE_TM_GMTOFF
#else
# define DONT_HAVE_TM_GMTOFF
#endif



/***** DO NOT CHANGE ANYTHING BELOW THIS LINE ********************************/

/*
 * Open modes for fopen()
 */
#define R_MODE		"r"
#define W_MODE		"w"
#define A_MODE		"a"
#define RP_MODE		"r+"
#define WP_MODE		"w+"
#define AP_MODE		"a+"


/*
 * Format strings for RFC dates
 */
#define DATE_DEFAULT	"%a, %d %b %Y %H:%M:%S %O"
#define DATE_NEWS	"%a, %d %b %Y %H:%M:%S %O"
#define DATE_MAIL	"%a, %d %b %Y %H:%M:%S %O (%Z)"



/*
 * Line width for formatted messages.
 */
#define MAX_LINELEN	80


/*
 * Program limits
 */
#define MAXADDRESS	32		/* Max. # of FTN address in CONFIG */
#define MAXDOSDRIVE	16		/* Max. # fo DOS drives in CONFIG */

#define MAXPATH		128		/* Max. size of path names */
#define MAXINETADDR	128		/* Max. size of an Internet address */
#define MAXUSERNAME	128		/* Max. size of an user name */
