/*:ts=8*/
/*****************************************************************************
 * FIDOGATE --- Gateway software UNIX <-> FIDO
 *
 * $Id: fidogate.h,v 3.7.1.1 1994/05/07 08:20:41 mj Exp mj $
 *
 * Common header file
 *
 *****************************************************************************
 * Copyright (C) 1990, 1993
 *  _____ _____
 * |     |___  |   Martin Junius             FIDO:      2:242/6.1
 * | | | |   | |   Republikplatz 3           Internet:  mj@sungate.fido.de
 * |_|_|_|@home|   D-52072 Aachen, Germany   Phone:     ++49-241-86931 (voice)
 *
 * This file is part of FIDOGATE.
 *
 * FIDOGATE is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * FIDOGATE is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with FIDOGATE; see the file COPYING.  If not, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************/

/*
 * Configuration header files
 */
#include "config.h"
#include "paths.h"		    /* Automatically generated by Makefile */


/*
 * heavy includin' ...
 */

/***** System *****/
#include <stdio.h>
#ifdef __STDC__
# include <stdlib.h>
# include <stdarg.h>
#else
# include <varargs.h>
#endif
#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include <errno.h>
#ifdef M_XENIX
# include <sys/ndir.h>
# define dirent direct
#else
# include <dirent.h>
#endif

/***** Local *****/
#include "node.h"			/* nodelist definitions */
#include "packet.h"			/* FIDO mail packet definition */
#include "sysexits.h"			/* EX_* defines */



/*
 * Defines
 */
#define TRUE	1
#define FALSE	0
#define OK	0
#define ERROR	(-1)

#define bool	int

/*
 * We supply our own version of the toupper()/tolower()
 * macros, because the exact behaviour of those in
 * <ctype.h> varies among systems.
 */
#undef _toupper
#undef _tolower
#undef toupper
#undef tolower

#define _toupper(c) ((c)-'a'+'A')
#define _tolower(c) ((c)-'A'+'a')
#define toupper(c)  (islower(c) ? _toupper(c) : (c))
#define tolower(c)  (isupper(c) ? _tolower(c) : (c))



/*
 * Function declarations
 */
#ifdef __STDC__
# define P(x)	x
#else
# define P(x)	()
#endif

/*
 * Function declarations for systems like SUNOS, where the standard
 * header files don't have them.
 */
#include "declare.h"


/*
 * Structure / type definitions
 */

/* Textlist */
typedef struct st_textline {
    char		*line;
    struct st_textline	*next;
    struct st_textline	*prev;
} Textline;

typedef struct st_textlist {
    Textline		*first;
    Textline		*last;
    unsigned long        n;
} Textlist;

/* RFCAddr */
typedef struct st_rfcaddr {
    /* user@addr (real) */
    char user[MAXUSERNAME];
    char addr[MAXINETADDR];
    char real[MAXUSERNAME];
    int  flags;
} RFCAddr;

#define ADDR_MAUS	1

    

/*
 * Function prototypes
 */

/* address.c */
extern int i_flag;
extern char address_error[];

int	parse_address		P((char *, char *, Node *));
int	isfido			P((void));
char   *ftn_to_inet		P((Node *, int));
Node   *inet_to_ftn		P((char *));
int	addr_is_local		P((char *));

/* aliases.c */
typedef struct st_alias {
    struct st_alias *next;
    Node node;
    char *username;
    char *fullname;
} Alias;

void	alias_init		P((void));
Alias  *alias_lookup		P((Node *, char *, char *));

/* areas.c */
typedef struct st_area {	/* Area/newsgroup entry with options */
    struct st_area *next;	    /* For linked list */
    char *area;			    /* FTN area */
    char *group;		    /* Newsgroup */
    int   zone;			    /* FTN zone AKA */
    char *origin;		    /* FTN origin line */
    char *distribution;		    /* Distribution */
    int   flags;		    /* Area/group flags, see AREA_xx defines */
} Area;

#define AREA_LOCALXPOST	1	/* -l  Only local crosspostings */
#define AREA_NOXPOST	2	/* -x  No crosspostings */
#define AREA_NOMAUS	4	/* -m  No messages from MAUSNET */
#define AREA_NOGATE	8	/* -g  No messages from other gateways */
#define AREA_8BIT	16	/* -8  Use 8 bit ISO-8859-1 character set */

void    areas_init		P((void));
Area   *areas_lookup		P((char *, char *));

/* binkley.c */
#define NOWAIT	0
#define WAIT	1

char   *bink_out_name		P((Node *));
char   *bink_bsy_name		P((Node *));
int	bink_bsy_test		P((Node *));
int	bink_bsy_create		P((Node *, int));
int	bink_bsy_delete		P((Node *));
char   *bink_find_flo		P((Node *, char *));
char   *bink_find_out		P((Node *, char *));
int	bink_attach		P((Node *, int, char *, char *));

/* bounce.c */
int	print_file_subst	P((FILE *, FILE*, Message*, char*, Textlist*));
int	bounce_header		P((char *, char *));
int	bounce_mail		P((char *, RFCAddr *, Message *,
				   char *, Textlist *));

/* bufstring.c */
char   *buf_sprintf		P((char *, ...));
char   *buf_copy		P((char *));

/* config.c */
void	cf_debug		P((void));
Node   *cf_addr			P((void));
Node   *cf_fake			P((void));
Node   *cf_uplink		P((void));
Node	cf_n_addr		P((void));
Node	cf_n_fake		P((void));
Node	cf_n_uplink		P((void));
void	cf_set_zone		P((int));
int	cf_zone			P(());
int	cf_defzone		P(());
char   *cf_getline		P((char *, int, FILE *));
void	cf_read_config_file	P((char *));
void	cf_initialize		P((void));
void	cf_set_addr		P((char *));
void	cf_set_fake		P((char *));
void	cf_set_uplink		P((char *));
void	cf_set_libdir		P((char *));
void	cf_set_spooldir		P((char *));
char   *cf_libdir		P((void));
char   *cf_spooldir		P((void));
char   *cf_hostname		P((void));
char   *cf_domainname		P((void));
char   *cf_hostsdomain		P((void));
char   *cf_fqdn			P((void));
char   *cf_origin		P((void));
char   *cf_organization		P((void));
void	cf_set_outbound		P((char *));
char   *cf_outbound		P((void));
char   *cf_zones_inet_domain	P((int));
int	cf_zones_check		P((int));
char   *cf_zones_trav		P((int));
char   *cf_zones_out		P((int));
Node   *cf_addr_trav		P((int));
int	cf_dos			P((void));
char   *cf_dos_xlate		P((char *));
void	cf_set_inbound		P((char *));
char   *cf_inbound		P((void));
Node    cf_gateway		P((void));

/* date.c */
char   *date			P((char *, time_t *));

/* fopen.c */
FILE   *xfopen			P((char *, char *));
FILE   *libfopen		P((char *, char *));
FILE   *spoolfopen		P((char *, char *));

/* gettime.c / parsedate.y */
typedef struct _TIMEINFO {
    time_t	time;
    long	usec;
    long	tzone;
} TIMEINFO;
extern time_t	parsedate	P((char *, TIMEINFO *));
extern int	GetTimeInfo	P((TIMEINFO *));

/* hosts.c */
typedef struct st_host {	/* hosts entry */
    struct st_host *next;	/*     for linked list */
    Node	node;		/*     FTN address */
    char       *name;		/*     Internet address */
    int		flags;		/*     flags */
} Host;

#define HOST_POINT	1	/* Addresses with pX point address */
#define HOST_DOWN	2	/* Temporary down */

void	hosts_init		P((void));
Host   *hosts_lookup		P((Node *, char *));

/* lock.c */
int	lock_fd			P((int));
int	unlock_fd		P((int));
int	lock_file		P((FILE *));
int	unlock_file		P((FILE *));

/* log.c */
extern int verbose;

char   *strerror		P((int));
void	log			P((const char *, ...));
void	debug			P((int, const char *, ...));

/* maus.c */
typedef struct st_maus {	/* MAUSNET domain list entry */
    struct st_maus *next;	    /* for linked list */
    char *name;			    /* MAUS system name */
    char *domain;		    /* Internet domain address */
    Node  gate;			    /* FTN gateway for MAUS system */
} Maus;

void	maus_init		P((void));
Maus   *maus_default		P((void));
Maus   *maus_lookup		P((char *));

/* message.c */
typedef struct st_body {
    /* structured FTN message body */
    char    *area;		/* AREA:xxxx echo tag */
    Textlist kludge;		/* ^A kludges at start of message */
    Textlist rfc;		/* RFC headers at start of message */
    Textlist body;		/* text body */
    char    *tear;		/* NetMail / EchoMail: --- tear line */
    char    *origin;		/* EchoMail:  * Origin: xxxx (addr) line */
    Textlist seenby;		/* EchoMail: SEEN-BY lines */
    Textlist path;		/* EchoMail: ^APATH lines */
    Textlist via;		/* NetMail: ^AVia lines */
} MsgBody;
    
int	pkt_get_line		P((FILE *, char *, int));
int	pkt_get_body		P((FILE *, Textlist *));
void	msg_body_init		P((MsgBody *));
void	msg_body_clear		P((MsgBody *));
int	msg_body_parse		P((Textlist *, MsgBody *));
void	msg_body_debug		P((FILE *, MsgBody *));

/* misc.c */
void	stripbad		P((char *));
char   *strncpy0		P((char *, char *, int));
char   *strncat0		P((char *, char *, int));
int	strnicmp		P((char *, char *, int));
int	stricmp			P((char *, char *));
long	xtol			P((char *));

/* msgid.c */
char   *msgid_fido_to_rfc	P((char *));
char   *msgid_default		P((Node *, char *, char *, char *, time_t));
char   *msgid_rfc_to_fido	P((int *, char *, int, int));
char   *msgid_rfc_to_origid	P((char *, int, int));
char   *msgid_convert_origid	P((char *, int));

/* node.c */
int	pfnz_to_node	        P((char *, Node *, int));
int	asc_to_node	        P((char *, Node *, int));
char   *node_to_asc	        P((Node *, int));
char   *node_to_pfnz	        P((Node *, int));
int	node_eq			P((Node *, Node *));
void	node_clear		P((Node *));

/* packet.c */
int	pkt_get_string		P((FILE *, char *, int));
time_t	pkt_get_date		P((FILE *));
int	pkt_get_msg_hdr		P((FILE *, Message *));
void	pkt_debug_msg_hdr	P((FILE *, Message *));
int	pkt_put_string		P((FILE *, char *));
int	pkt_put_line		P((FILE *, char *));
int	pkt_put_int16		P((FILE *, int));
int	pkt_put_date		P((FILE *, time_t));
int	pkt_put_msg_hdr		P((FILE *, Message *, int));
long	pkt_get_int16		P((FILE *));
int	pkt_get_nbytes		P((FILE *, char *, int));
int 	pkt_get_hdr		P((FILE *, Packet *));
void	pkt_debug_hdr		P((FILE *, Packet *));
int	pkt_put_string_padded	P((FILE *, char *, int));
int	pkt_put_hdr		P((FILE *, Packet *));

/* rfcaddr.c */
RFCAddr	rfcaddr_from_ftn	P((char *, Node *));
RFCAddr	rfcaddr_from_rfc	P((char *));
char   *rfcaddr_to_asc		P((RFCAddr *, int));

/* rfcheader.c */
void	header_delete		P((void));
void	header_read		P((FILE *));
char   *header_get		P((char *));
char   *header_getnext		P((void));
char   *header_getcomplete	P((char *));
char   *name_from_rfcaddr	P((char *, char *));
char   *addr_from_rfcaddr	P((char *, char *));
char   *username_from_rfcaddr	P((char *, char *));
char   *addr_token		P((char *));

/* sequencer.c */
long	sequencer		P((char *));

/* xalloc.c */
#define BUFFERSIZE		(16*1024)	/* Global buffer */
extern char buffer[BUFFERSIZE];

char   *xmalloc			P((int));
char   *xrealloc		P((char *, int));
char   *strsave			P((char *));

/* xstrtok.c --- module by Wolfram Roesler */
char   *xstrtok			P((char *,char *));
char   *strsaveline		P((char *));

/* crc32.c */
unsigned long compute_crc32 P((unsigned char *, int));
void	crc32_init	    P((void));
void	crc32_compute	    P((unsigned char *, int));
unsigned long crc32_value   P((void));

/* textlist.c */
void	tl_add			P((Textlist *, Textline *));
void	tl_remove		P((Textlist *, Textline *));
void	tl_delete		P((Textlist *, Textline *));
void	Textlist_init		P((Textlist *));
void	Textlist_append		P((Textlist *, char *));
void	Textlist_appendf	P((Textlist *, char *, ...));
void	Textlist_print		P((Textlist *, FILE *));
void	Textlist_clear		P((Textlist *));
long	Textlist_size		P((Textlist *));
void	Textlist_add		P((Textlist *, Textlist *));

/* charset.c */
void	charset_reset	    P((void));
void	charset_set	    P((char *));
char   *charset_xlate	    P((int));

/* version.c */
char   *version_global		P((void));
char   *version_local		P((char *));
int	version_major		P((void));
int	version_minor		P((void));

/* read.c */
char *read_line		    P((char *, int, FILE *));
long  read_rnews_size	    P((FILE *));

/* wildmat */
int	wildmat			P((char *, char *));

/* getdate.y */
time_t	get_date		P((char *, void *));

/* mail.c */
extern char mail_dir[];

int	mail_open		P((void));
char   *mail_name		P((void));
FILE   *mail_file		P((void));
int	mail_kill		P((void));
int	mail_close		P((void));
